{ Demo program for ChroMathsome - Linear Algebra Pro.
  Below, you find example implementations of using the system of equation solver(s)
  from Linear Algebra Pro.

  This code was written during the development stages of ChroMathsome - Linear Algebra.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) and some of the variables a little strange.

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), we gave most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!
  In this demo, we used our privilege to ignore this :-))

  Read the help file for information about the implemented functions
  and their limitations/possibilities!!!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "C".
  "Progress has no patience - but something's got to give"              }

unit Syseq;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls, LinAlg, LinAlgP;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    ClearMemo1: TMenuItem;
    Close1: TMenuItem;
    Real1: TMenuItem;
    Complex1: TMenuItem;
    Exact1: TMenuItem;
    UnderDetermined1: TMenuItem;
    OverDetermined1: TMenuItem;
    SolveSystems1: TMenuItem;
    procedure ClearMemo1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Exact1Click(Sender: TObject);
    procedure UnderDetermined1Click(Sender: TObject);
    procedure OverDetermined1Click(Sender: TObject);
    procedure SolveSystems1Click(Sender: TObject);
  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_RealVector(V: RealVector);
    procedure Write_ComplexVector(V: ComplexVector);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-17 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,8,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Write a complex matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-17 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-17 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

{ Write a real vector to the screen, creating a nice readable look. }
procedure TForm1.Write_RealVector(V: RealVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if abs(V.Vector^[a]) < 1E-17 then
    begin
    V.Vector^[a]:= 0;
    matt:= IntToStr(round(V.Vector^[a]));
    end
    else
    matt:= FloatToStrF(V.Vector^[a], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexVector(V: ComplexVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    matt:= '(' + FloatToStr(V.Vector^[a].re) + ';' +  FloatToStr(V.Vector^[a].im) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;


procedure TForm1.ClearMemo1Click(Sender: TObject);
begin
     if MessageDlg('Delete the contents of this memo?',
     mtInformation, [mbYes, mbNo], 0) = mrYes
     then
     Memo1.Lines.Clear;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

{ Let's solve some sample systems of linear equations.
  Notice the resulting MATRIX!
  Read the help file for information on how to interprete this matrix.

  A. Exact determined systems i.e. # of equations = # of variables.
  Solve some (non-) homogeneous systems.
  Note: we will also calculate the residual vectors.
        This gives an idea about the accuracy of the solution(s).            }
procedure TForm1.Exact1Click(Sender: TObject);
var M, CoM: RealMatrix;
    CoV, ResV, X, V1: RealVector;
    NumOfSolutions, i, j: integer;
begin
     InitRealMatrix(M, 3, 3);
     InitRealMatrix(CoM, 3, 3);
     InitRealVector(CoV, 3);
     InitRealVector(V1, 3);
     InitRealVector(ResV, 3);
     InitRealVector(X, 3);
     try
     { Example 1: Simple 3 * 3 system -> infinitely many solutions besides the trivial one (0, 0, 0). }
     CoM.Matrix^[1]^[1]:= -18;
     CoM.Matrix^[1]^[2]:= -57;
     CoM.Matrix^[1]^[3]:= -69;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 4;
     CoM.Matrix^[2]^[3]:= 3;
     CoM.Matrix^[3]^[1]:= 5;
     CoM.Matrix^[3]^[2]:= 15;
     CoM.Matrix^[3]^[3]:= 20;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 2: Simple 3 * 3 system -> only one solution: (0, 0, 0). }
     CoM.Matrix^[1]^[1]:= 3;
     CoM.Matrix^[1]^[2]:= -2;
     CoM.Matrix^[1]^[3]:= 2;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 2;
     CoM.Matrix^[2]^[3]:= -3;
     CoM.Matrix^[3]^[1]:= 4;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 2;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

      { Example 3: Simple 3 * 3 system -> infinitely many solutions besides the trivial (0, 0, 0). }
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 4: Simple 3 * 3 system -> infinitely many solutions. }
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 2;
     CoV.Vector^[3]:= 12;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

      { Example 5: Simple 3 * 3 system -> no solution(s). }
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 2;
     CoV.Vector^[3]:= -4;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

      { Example 6: Simple 6 * 6 system -> 1 solution. }
     Redimension_RealMatrix(CoM, 6, 6);
     Redimension_RealVector(CoV, 6);
     CoM.Matrix^[1]^[1]:= 0;
     CoM.Matrix^[1]^[2]:= 0;
     CoM.Matrix^[1]^[3]:= 0;
     CoM.Matrix^[1]^[4]:= 1;
     CoM.Matrix^[1]^[5]:= -1;
     CoM.Matrix^[1]^[6]:= 1;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -1;
     CoM.Matrix^[2]^[3]:= 0;
     CoM.Matrix^[2]^[4]:= 0;
     CoM.Matrix^[2]^[5]:= 1;
     CoM.Matrix^[2]^[6]:= 0;
     CoM.Matrix^[3]^[1]:= -1;
     CoM.Matrix^[3]^[2]:= 0;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[3]^[4]:= 0;
     CoM.Matrix^[3]^[5]:= 0;
     CoM.Matrix^[3]^[6]:= -1;
     CoM.Matrix^[4]^[1]:= 0;
     CoM.Matrix^[4]^[2]:= -2;
     CoM.Matrix^[4]^[3]:= 0;
     CoM.Matrix^[4]^[4]:= -3;
     CoM.Matrix^[4]^[5]:= -4;
     CoM.Matrix^[4]^[6]:= 0;
     CoM.Matrix^[5]^[1]:= 4;
     CoM.Matrix^[5]^[2]:= 2;
     CoM.Matrix^[5]^[3]:= 3;
     CoM.Matrix^[5]^[4]:= 0;
     CoM.Matrix^[5]^[5]:= 0;
     CoM.Matrix^[5]^[6]:= 0;
     CoM.Matrix^[6]^[1]:= -4;
     CoM.Matrix^[6]^[2]:= 0;
     CoM.Matrix^[6]^[3]:= 0;
     CoM.Matrix^[6]^[4]:= 0;
     CoM.Matrix^[6]^[5]:= 4;
     CoM.Matrix^[6]^[6]:= 6;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     CoV.Vector^[4]:= 6;
     CoV.Vector^[5]:= -2;
     CoV.Vector^[6]:= 20;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(CoM);
     DestroyRealVector(CoV);
     DestroyRealVector(ResV);
     DestroyRealVector(X);
     DestroyRealVector(V1);
     end;
end;

{ B. Under-determined systems i.e. more variables than equations.
     Note: these systems usually have infinitely many solutions.   }
procedure TForm1.UnderDetermined1Click(Sender: TObject);
var M, CoM: RealMatrix;
    CoV, ResV, X, V1: RealVector;
    NumOfSolutions, i, j: integer;
begin
     InitRealMatrix(M, 3, 3);
     InitRealMatrix(CoM, 3, 3);
     InitRealVector(CoV, 3);
     InitRealVector(V1, 3);
     InitRealVector(ResV, 3);
     InitRealVector(X, 3);
     try
     { Example 1: 4 * 5 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 4, 5);
     Redimension_RealVector(CoV, 4);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[1]^[3]:= -2;
     CoM.Matrix^[1]^[4]:= -1;
     CoM.Matrix^[1]^[5]:= 0;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 3;
     CoM.Matrix^[2]^[3]:= -5;
     CoM.Matrix^[2]^[4]:= 1;
     CoM.Matrix^[2]^[5]:= -7;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[3]^[4]:= -1;
     CoM.Matrix^[3]^[5]:= 1;
     CoM.Matrix^[4]^[1]:= -1;
     CoM.Matrix^[4]^[2]:= 1;
     CoM.Matrix^[4]^[3]:= 5;
     CoM.Matrix^[4]^[4]:= -1;
     CoM.Matrix^[4]^[5]:= 0;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 4;
     CoV.Vector^[3]:= 2;
     CoV.Vector^[4]:= 3;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 2: 3 * 4 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 3, 4);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= -1;
     CoM.Matrix^[1]^[3]:= 5;
     CoM.Matrix^[1]^[4]:= -1;
     CoM.Matrix^[2]^[1]:= -2;
     CoM.Matrix^[2]^[2]:= 1;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[2]^[4]:= -1;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 2;
     CoM.Matrix^[3]^[3]:= -1;
     CoM.Matrix^[3]^[4]:= -1;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= -5;
     CoV.Vector^[3]:= -2;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

      { Example 3: 3 * 5 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 3, 5);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= -5;
     CoM.Matrix^[1]^[3]:= 3;
     CoM.Matrix^[1]^[4]:= -4;
     CoM.Matrix^[1]^[5]:= 2;
     CoM.Matrix^[2]^[1]:= 3;
     CoM.Matrix^[2]^[2]:= -7;
     CoM.Matrix^[2]^[3]:= 2;
     CoM.Matrix^[2]^[4]:= -5;
     CoM.Matrix^[2]^[5]:= 4;
     CoM.Matrix^[3]^[1]:= 5;
     CoM.Matrix^[3]^[2]:= -10;
     CoM.Matrix^[3]^[3]:= -5;
     CoM.Matrix^[3]^[4]:= -4;
     CoM.Matrix^[3]^[5]:= 7;

     CoV.Vector^[1]:= 4;
     CoV.Vector^[2]:= 9;
     CoV.Vector^[3]:= 22;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

       { Example 4: 3 * 5 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 3, 5);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 3;
     CoM.Matrix^[1]^[3]:= -2;
     CoM.Matrix^[1]^[4]:= 5;
     CoM.Matrix^[1]^[5]:= -3;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 7;
     CoM.Matrix^[2]^[3]:= -3;
     CoM.Matrix^[2]^[4]:= 7;
     CoM.Matrix^[2]^[5]:= -5;
     CoM.Matrix^[3]^[1]:= 3;
     CoM.Matrix^[3]^[2]:= 11;
     CoM.Matrix^[3]^[3]:= -4;
     CoM.Matrix^[3]^[4]:= 10;
     CoM.Matrix^[3]^[5]:= -9;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 5: 4 * 5 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 4, 5);
     Redimension_RealVector(CoV, 4);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[1]^[4]:= -1;
     CoM.Matrix^[1]^[5]:= 0;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 3;
     CoM.Matrix^[2]^[3]:= -5;
     CoM.Matrix^[2]^[4]:= 1;
     CoM.Matrix^[2]^[5]:= -7;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[3]^[4]:= -1;
     CoM.Matrix^[3]^[5]:= 1;
     CoM.Matrix^[4]^[1]:= -1;
     CoM.Matrix^[4]^[2]:= 1;
     CoM.Matrix^[4]^[3]:= 5;
     CoM.Matrix^[4]^[4]:= -1;
     CoM.Matrix^[4]^[5]:= 0;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 4;
     CoV.Vector^[3]:= 2;
     CoV.Vector^[4]:= 3;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 6: 3 * 4 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 3, 4);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= -1;
     CoM.Matrix^[1]^[3]:= 5;
     CoM.Matrix^[1]^[4]:= -4;
     CoM.Matrix^[2]^[1]:= -2;
     CoM.Matrix^[2]^[2]:= 1;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[2]^[4]:= -1;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 2;
     CoM.Matrix^[3]^[3]:= -1;
     CoM.Matrix^[3]^[4]:= -1;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= -5;
     CoV.Vector^[3]:= -2;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(CoM);
     DestroyRealVector(CoV);
     DestroyRealVector(ResV);
     DestroyRealVector(X);
     DestroyRealVector(V1);
     end;
end;

{ B. Over-determined systems i.e. more variables than equations.
     Note: these systems usually don't have a solution.
     The general system solver will return a value of 0 for the NumOfSolutions variable in
     99% of over-determined systems.
     If so, use the least squares solver for this type of systems.        }
procedure TForm1.OverDetermined1Click(Sender: TObject);
var M, CoM: RealMatrix;
    CoV, ResV, X, V1: RealVector;
    NumOfSolutions, i, j: integer;
begin
     InitRealMatrix(M, 3, 3);
     InitRealMatrix(CoM, 3, 3);
     InitRealVector(CoV, 3);
     InitRealVector(V1, 3);
     InitRealVector(ResV, 3);
     InitRealVector(X, 3);
     try
     { Example 1: 4 * 3 system . }
     Redimension_RealMatrix(CoM, 4, 3);
     Redimension_RealVector(CoV, 4);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[1]^[3]:= -1;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 5;
     CoM.Matrix^[2]^[3]:= 2;
     CoM.Matrix^[3]^[1]:= 1;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= 7;
     CoM.Matrix^[4]^[1]:= 1;
     CoM.Matrix^[4]^[2]:= 3;
     CoM.Matrix^[4]^[3]:= 3;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     CoV.Vector^[4]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

      RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then
      begin
      Write_Message('System has no solutions!');
      RealSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_RealVector(X);
      end
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 2: 3 * 2 system. }
     Redimension_RealMatrix(CoM, 3, 2);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= -1;
     CoM.Matrix^[3]^[1]:= 3;
     CoM.Matrix^[3]^[2]:= -4;

     CoV.Vector^[1]:= 4;
     CoV.Vector^[2]:= 3;
     CoV.Vector^[3]:= 2;

     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then
      begin
      Write_Message('System has no solutions!');
      RealSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_RealVector(X);
      end
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 3: 4 * 3 system. }
     Redimension_RealMatrix(CoM, 4, 3);
     Redimension_RealVector(CoV, 4);
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 0;
     CoM.Matrix^[1]^[3]:= -1;
     CoM.Matrix^[2]^[1]:= 0;
     CoM.Matrix^[2]^[2]:= 1;
     CoM.Matrix^[2]^[3]:= 2;
     CoM.Matrix^[3]^[1]:= 1;
     CoM.Matrix^[3]^[2]:= 0;
     CoM.Matrix^[3]^[3]:= 1;
     CoM.Matrix^[4]^[1]:= 5;
     CoM.Matrix^[4]^[2]:= 1;
     CoM.Matrix^[4]^[3]:= 1;

     CoV.Vector^[1]:= 1;
     CoV.Vector^[2]:= -2;
     CoV.Vector^[3]:= 2;
     CoV.Vector^[4]:= 1;

     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

      RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then
      begin
      Write_Message('System has no solutions!');
      RealSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_RealVector(X);
      end
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     { Example 4: 5 * 1 system. }
     Redimension_RealMatrix(CoM, 5, 1);
     Redimension_RealVector(CoV, 5);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[3]^[1]:= 3;
     CoM.Matrix^[4]^[1]:= 4;
     CoM.Matrix^[5]^[1]:= 5;

     CoV.Vector^[1]:= 1;
     CoV.Vector^[2]:= 1;
     CoV.Vector^[3]:= 1;
     CoV.Vector^[4]:= 1;
     CoV.Vector^[5]:= 1;

     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then
      begin
      Write_Message('System has no solutions!');
      RealSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_RealVector(X);
      end
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

    { Example 5: 3 * 2 system. }
     Redimension_RealMatrix(CoM, 3, 2);
     Redimension_RealVector(CoV, 3);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 3;
     CoM.Matrix^[3]^[1]:= 0;
     CoM.Matrix^[3]^[2]:= 0;

     CoV.Vector^[1]:= 4;
     CoV.Vector^[2]:= 5;
     CoV.Vector^[3]:= 6;

     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

      RealSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then
      begin
      Write_Message('System has no solutions!');
      RealSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_RealVector(X);
      end
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_RealMatrix(M);
      if NumOfSolutions = 1 then
      begin
      RealMatrixToRealVector(X, M);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(RealVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilRealVector(V1);
      for j:= 1 to i do
      begin
      RealMatrixRowToRealVector(X, M, j);
      Redimension_RealVector(V1, X.Dimension);
      RealVector_Addition(X, V1, X);
      RealSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_RealVector(ResV);
      Copy_RealVector(V1, X);
      end;
      end;
      end;

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(CoM);
     DestroyRealVector(CoV);
     DestroyRealVector(ResV);
     DestroyRealVector(X);
     DestroyRealVector(V1);
     end;
end;

{ Complex systems.
  I could only find 2 examples ...
  (could of course "create" more but I'm too lazy ;-)) ).    }
procedure TForm1.SolveSystems1Click(Sender: TObject);
var M, CoM: ComplexMatrix;
    CoV, ResV, X, V1: ComplexVector;
    NumOfSolutions, i, j: integer;
begin
     InitComplexMatrix(M, 4, 4);
     InitComplexMatrix(CoM, 4, 4);
     InitComplexVector(CoV, 4);
     InitComplexVector(V1, 4);
     InitComplexVector(ResV, 4);
     InitComplexVector(X, 4);
     try
     { Example 1: 3 * 4 system -> infinitely many solutions. }
     Redimension_ComplexMatrix(CoM, 3, 4);
     Redimension_ComplexVector(CoV, 3);
     CoM.Matrix^[1]^[1].re:= 0;
     CoM.Matrix^[1]^[1].im:= 0;
     CoM.Matrix^[1]^[2].re:= 1;
     CoM.Matrix^[1]^[2].im:= 0;
     CoM.Matrix^[1]^[3].re:= 0;
     CoM.Matrix^[1]^[3].im:= 1;
     CoM.Matrix^[1]^[4].re:= -1;
     CoM.Matrix^[1]^[4].im:= 0;
     CoM.Matrix^[2]^[1].re:= 1;
     CoM.Matrix^[2]^[1].im:= 0;
     CoM.Matrix^[2]^[2].re:= 0;
     CoM.Matrix^[2]^[2].im:= 1;
     CoM.Matrix^[2]^[3].re:= -1;
     CoM.Matrix^[2]^[3].im:= 0;
     CoM.Matrix^[2]^[4].re:= 0;
     CoM.Matrix^[2]^[4].im:= -1;
     CoM.Matrix^[3]^[1].re:= 1;
     CoM.Matrix^[3]^[1].im:= 0;
     CoM.Matrix^[3]^[2].re:= 1;
     CoM.Matrix^[3]^[2].im:= 0;
     CoM.Matrix^[3]^[3].re:= 0;
     CoM.Matrix^[3]^[3].im:= 1;
     CoM.Matrix^[3]^[4].re:= 1;
     CoM.Matrix^[3]^[4].im:= 0;

     CoV.Vector^[1].re:= -1;
     CoV.Vector^[1].im:= 0;
     CoV.Vector^[2].re:= 0;
     CoV.Vector^[2].im:= 0;
     CoV.Vector^[3].re:= 0;
     CoV.Vector^[3].im:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_ComplexMatrix(CoM);
     Write_Message('Y: ');
     Write_ComplexVector(CoV);

     ComplexSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_ComplexMatrix(M);
      if NumOfSolutions = 1 then
      begin
      ComplexMatrixToComplexVector(X, M);
      ComplexSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_ComplexVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(ComplexVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilComplexVector(V1);
      for j:= 1 to i do
      begin
      ComplexMatrixRowToComplexVector(X, M, j);
      Redimension_ComplexVector(V1, X.Dimension);
      ComplexVector_Addition(X, V1, X);
      ComplexSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_ComplexVector(ResV);
      Copy_ComplexVector(V1, X);
      end;
      end;
      end;

     { Example 2: 4 * 4 system -> one solution. }
     Redimension_ComplexMatrix(CoM, 4, 4);
     Redimension_ComplexVector(CoV, 4);
     CoM.Matrix^[1]^[1].re:= 1;
     CoM.Matrix^[1]^[1].im:= 0;
     CoM.Matrix^[1]^[2].re:= 0;
     CoM.Matrix^[1]^[2].im:= 1;
     CoM.Matrix^[1]^[3].re:= 0;
     CoM.Matrix^[1]^[3].im:= 0;
     CoM.Matrix^[1]^[4].re:= 0;
     CoM.Matrix^[1]^[4].im:= 1;
     CoM.Matrix^[2]^[1].re:= 0;
     CoM.Matrix^[2]^[1].im:= 1;
     CoM.Matrix^[2]^[2].re:= 1;
     CoM.Matrix^[2]^[2].im:= 0;
     CoM.Matrix^[2]^[3].re:= 0;
     CoM.Matrix^[2]^[3].im:= 1;
     CoM.Matrix^[2]^[4].re:= 0;
     CoM.Matrix^[2]^[4].im:= 0;
     CoM.Matrix^[3]^[1].re:= 0;
     CoM.Matrix^[3]^[1].im:= 0;
     CoM.Matrix^[3]^[2].re:= 0;
     CoM.Matrix^[3]^[2].im:= 1;
     CoM.Matrix^[3]^[3].re:= 1;
     CoM.Matrix^[3]^[3].im:= 0;
     CoM.Matrix^[3]^[4].re:= 0;
     CoM.Matrix^[3]^[4].im:= 1;
     CoM.Matrix^[4]^[1].re:= 0;
     CoM.Matrix^[4]^[1].im:= 1;
     CoM.Matrix^[4]^[2].re:= 0;
     CoM.Matrix^[4]^[2].im:= 0;
     CoM.Matrix^[4]^[3].re:= 0;
     CoM.Matrix^[4]^[3].im:= 1;
     CoM.Matrix^[4]^[4].re:= 1;
     CoM.Matrix^[4]^[4].im:= 0;

     CoV.Vector^[1].re:= -1;
     CoV.Vector^[1].im:= 0;
     CoV.Vector^[2].re:= 0;
     CoV.Vector^[2].im:= 2;
     CoV.Vector^[3].re:= 0;
     CoV.Vector^[3].im:= 0;
     CoV.Vector^[4].re:= 0;
     CoV.Vector^[4].im:= 1;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_ComplexMatrix(CoM);
     Write_Message('Y: ');
     Write_ComplexVector(CoV);

     ComplexSysOfEquations(M, CoM, CoV, NumOfSolutions);
      if NumOfSolutions = 0 then Write_Message('System has no solutions!')
      else
      begin
      if NumOfSolutions = 1 then Write_Message('Unique solution of system: ')
      else Write_Message('Solutions of system: ');
      Write_ComplexMatrix(M);
      if NumOfSolutions = 1 then
      begin
      ComplexMatrixToComplexVector(X, M);
      ComplexSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_ComplexVector(ResV);
      (*
      { Eventually, calculate the row norm of ResV to find the largest absolute value ... }
      Write_Message('row norm of residual vector: ' + FloatToStr(ComplexVector_RowNorm(ResV))); *)
      end
      else
      begin
      i:= M.Row;
      NilComplexVector(V1);
      for j:= 1 to i do
      begin
      ComplexMatrixRowToComplexVector(X, M, j);
      Redimension_ComplexVector(V1, X.Dimension);
      ComplexVector_Addition(X, V1, X);
      ComplexSysOfEquationsAccuracyTest(ResV, CoM, X, CoV);
      Write_Message('Residual vector: ');
      Write_ComplexVector(ResV);
      Copy_ComplexVector(V1, X);
      end;
      end;
      end;
      { Note: there's also a least squares solver for complex systems.
        As an example, we'll calculate the least squares solution of the previous system.
        If the system has one unique solution, it's obvious that this is also the least squares solution. }
      ComplexSysOfEquationsLeastSquare(X, CoM, CoV);
      Write_Message('Least squares solution: ');
      Write_ComplexVector(X);


     finally
     DestroyComplexMatrix(M);
     DestroyComplexMatrix(CoM);
     DestroyComplexVector(CoV);
     DestroyComplexVector(ResV);
     DestroyComplexVector(X);
     DestroyComplexVector(V1);
     end;
end;

end.
